RunScript("system/player_utils.lua");

function SetDefaults()

--//entity stuff
	this:SetCategory(C_CATEGORY_ENTITY, false);
	this:SetCategory(C_CATEGORY_PLAYER, true);
	this:SetCollisionListenCategory(C_CATEGORY_TILEPIC, true);
	this:SetCollisionListenCategory(C_CATEGORY_ENTITY, true);
	this:GetBrainManager():Add("StandardBase","");
	--//this:GetBrainManager():SendToBrainBase("control_filtery=0.5");
	
	m_baseSpeed = 5;
	this:SetDesiredSpeed(m_baseSpeed);
	this:SetTurnSpeed(1.0);
	this:SetDampening(0.1);
	m_delayBetweenAttacksMS = 500;
	m_attackTimer = 0;
	HasSpell = this:Data():SetIfNull("HasSpell", "none");
	
	ResetKeys()
	
end

function SetupInputKeys()

--//keyboard
	--//movement
	GetInputManager:AddBinding("a,always", "OnLeft", this:GetID());
	GetInputManager:AddBinding("d,always", "OnRight", this:GetID());
	GetInputManager:AddBinding("w,always", "OnUp", this:GetID());
	GetInputManager:AddBinding("s,always", "OnDown", this:GetID());
	--//actions
	GetInputManager:AddBinding("control", "OnAttackButton", this:GetID());
	GetInputManager:AddBinding("e,always", "OnActionButton", this:GetID());

--//joystick
	--//movement
	GetInputManager:AddBinding("joy_any_left,always", "OnLeft", this:GetID());
	GetInputManager:AddBinding("joy_any_right,always", "OnRight", this:GetID());
	GetInputManager:AddBinding("joy_any_up,always", "OnUp", this:GetID());
	GetInputManager:AddBinding("joy_any_down,always", "OnDown", this:GetID());
	--//actions
	GetInputManager:AddBinding("joy_any_button_3", "OnAttackButton", this:GetID());
	GetInputManager:AddBinding("joy_any_button_0", "OnActionButton", this:GetID());

end

--//check for pressed buttons
function OnLeft(bKeyDown)

	m_bLeft = bKeyDown;
	return true;
end

function OnRight(bKeyDown)

	m_bRight = bKeyDown;
	return true;
end

function OnDown(bKeyDown)

	m_bDown = bKeyDown;
	return true;
end

function OnUp(bKeyDown)

	m_bUp = bKeyDown;
	return true;
end

function OnAttackButton(bKeyDown)

	if (bKeyDown) then
	
		if (m_attackTimer < GetApp:GetGameTick()) then
			m_bAttack = true;
			m_attackTimer = GetApp:GetGameTick() + m_delayBetweenAttacksMS;
		end
	end	
	
	return true; 	
end

function OnActionButton(bKeyDown)

	if (bKeyDown) then
		m_bAction = true;
	end
	
	return true;
end

function ResetKeys()

	m_bLeft = false;
	m_bRight = false;
	m_bUp = false
	m_bDown = false;
	
	m_bAttack = false;
	m_bAction = false;
	
end

function ConvertKeysToIso(bLeft, bRight, bUp, bDown)

		local speedMod = 1; --//normal

	
	if (bLeft == false and bRight == false and bUp == false and bDown == false) then
		--//no keys are pressed
		return C_FACING_NONE, Vector2(0,0), speedMod;
	end
	
	local facing;
		
	if (bLeft) then
		facing = C_FACING_UP_LEFT;		
	end

	if (bRight) then
		facing = C_FACING_DOWN_RIGHT;		
	end

	if (bDown) then
		facing = C_FACING_DOWN_LEFT;
	end

	if (bUp) then
		facing = C_FACING_UP_RIGHT;
	end

	--//also check for diagonals
	
	if (bDown and bLeft) then
		facing = C_FACING_LEFT;

	elseif (bUp and bLeft) then
		facing = C_FACING_UP;

	elseif (bDown and bRight) then
		facing = C_FACING_DOWN;

	elseif (bUp and bRight) then
		facing = C_FACING_RIGHT;
	end

	local v = FacingToVector(facing); --//convert to a vector
	

	if (facing == C_FACING_UP_RIGHT or facing == C_FACING_DOWN_LEFT) then
		--//fix it so the character walks exactly on the line we want, not sure why we need to do this
	
		local angle = math.atan2(v.x,v.y);
		
		--//rotate just a bit clockwise
		angle = angle + -0.32175;
		
		--//convert back to a vector
		v = Vector2(math.sin(angle), math.cos(angle));
	end

	if (facing == C_FACING_DOWN_RIGHT or facing == C_FACING_UP_LEFT) then
		--//fix it so the character walks exactly on the line we want, not sure why we need to do this
	
		local angle = math.atan2(v.x,v.y);
		
		--//rotate just a bit clockwise
		angle = angle + 0.32175;
		
		--//convert back to a vector
		v = Vector2(math.sin(angle), math.cos(angle));

	end


	if (facing == C_FACING_UP or facing == C_FACING_DOWN) then
			speedMod = 0.7; --//can't move as fast this way
	end

	return facing, v, speedMod;
end

function OnInit()

	this:SetVisualProfile("~/iso-player.xml", "isoplayer");
	this:LoadCollisionInfo("~/iso-player.col");
	
end

function OnPostInit()

	this:SetName("Player");
	SetDefaults();
	SetupInputKeys();
	this:SetRunUpdateEveryFrame(true);
	
end

function Update(step)
	
	AssignPlayerToCameraIfNeeded(this);
	
	if (this:GetGoalManager():GetGoalCount() == 0) then
		
		if (this:GetBrainManager():InState("Attack") == false ) then
			ProcessMovement();
		end
		
		--if (m_bAction) then	ProcessAction(); end; --//they hit the action key
		if (m_bAttack) then	ProcessAttack(); end; --//they hit the attack key
	
	else
		
	end
	
end

function ProcessMovement()

	local facing, vDir, speedMod = ConvertKeysToIso(m_bLeft, m_bRight, m_bUp, m_bDown);
	
	LogMsg("Vector is", vDir, "Speed mod:", speedMod);
	this:SetDesiredSpeed(m_baseSpeed*speedMod); --//if we wanted custom control of speed per dir
	if (facing != C_FACING_NONE) then
		--//they are pressing towards a direction
		this:SetVectorFacingTarget(vDir); --//turn the way we want
		this:GetBrainManager():SetStateByName("Walk");
	else
		this:GetBrainManager():SetStateByName("Idle");
	end
	
end

function ProcessAttack()

	if (HasSpell != "none") then
	this:GetBrainManager():SetStateByName("Attack");
	
	
	Schedule(200, this:GetID(), 'ent = CreateEntity(this:GetMap(), this:GetPos() + (FacingToVector(this:GetFacing()) * 32 + Vector2(0,-16)), "spells/'..HasSpell..'/'..HasSpell..'.lua"); ent:RunFunction("SetParent", this); ');
	
	end
	
	
	m_bAttack = false;

end

function OnAttackLoop()

	this:GetBrainManager():SetStateByName("Idle");
end

function SpellEquip(name)
	this:Data():Set("HasSpell", name);
	HasSpell = name;

end


function OnCollision(vPosition, vVelocity, vNormal, depth, materialID, entity, state)
  
return;
end
